--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

-- Purpose:
-- . Wrapper for corr_folder that allows the use of t_dp_sosi_2arr_2
-- Description:
-- . Our input is an array of                             g_nof_inputs*2 sosi buses
-- . Our 2 corr_folder instances below accept an array of 2*g_nof_inputs sosi buses
-- => We need to do a tranposed signal assignment.

ENTITY corr_folder_2arr_2 IS
  GENERIC (
    g_nof_inputs : NATURAL;
    g_nof_folds  : INTEGER := -1
  );                             
  PORT ( 
    rst            : IN  STD_LOGIC;
    clk            : IN  STD_LOGIC;

    snk_in_2arr_2  : IN  t_dp_sosi_2arr_2(g_nof_inputs-1 DOWNTO 0);

    src_out_2arr_2 : OUT t_dp_sosi_2arr_2(sel_a_b(g_nof_folds>=0, ceil_div(g_nof_inputs, ceil_pow2(g_nof_folds)), 1)-1 DOWNTO 0)
  );
END corr_folder_2arr_2;

ARCHITECTURE str OF corr_folder_2arr_2 IS

  CONSTANT c_nof_outputs : NATURAL := sel_a_b(g_nof_folds>=0, ceil_div(g_nof_inputs, ceil_pow2(g_nof_folds)), 1);

  TYPE t_corr_folder_snk_in_2arr IS ARRAY (2-1 DOWNTO 0) OF t_dp_sosi_arr(g_nof_inputs-1 DOWNTO 0);
  SIGNAL corr_folder_snk_in_2arr  : t_corr_folder_snk_in_2arr;

  TYPE t_corr_folder_src_out_2arr IS ARRAY (2-1 DOWNTO 0) OF t_dp_sosi_arr(c_nof_outputs-1 DOWNTO 0);
  SIGNAL corr_folder_src_out_2arr : t_corr_folder_src_out_2arr;

BEGIN

  ------------------------------------------------------------------------------
  -- Folder input: tranposed entity input
  ------------------------------------------------------------------------------
  gen_input_i : FOR i IN 0 TO g_nof_inputs-1 GENERATE
    gen_input_j : FOR j IN 0 TO 2-1 GENERATE
      corr_folder_snk_in_2arr(j)(i) <= snk_in_2arr_2(i)(j);
    END GENERATE;
  END GENERATE;

  ------------------------------------------------------------------------------
  -- Our two folders
  ------------------------------------------------------------------------------
  gen_corr_folder: FOR i IN 0 TO 2-1 GENERATE
    u_corr_folder : ENTITY work.corr_folder
    GENERIC MAP (
      g_nof_inputs => g_nof_inputs,
      g_nof_folds  => g_nof_folds 
    )
    PORT MAP (
      rst         => rst,
      clk         => clk,
  
      snk_in_arr  => corr_folder_snk_in_2arr(i),
  
      src_out_arr => corr_folder_src_out_2arr(i)
    );
  END GENERATE;

  ------------------------------------------------------------------------------
  -- Entity output: tranposed folder output
  ------------------------------------------------------------------------------
  gen_output_i : FOR i IN 0 TO c_nof_outputs-1 GENERATE
    gen_output_j : FOR j IN 0 TO 2-1 GENERATE
      src_out_2arr_2(i)(j) <= corr_folder_src_out_2arr(j)(i);
    END GENERATE;
  END GENERATE;

END str;

