-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose : Remote update from flash device

LIBRARY ieee, technology_lib;
USE ieee.std_logic_1164.all;
USE work.tech_flash_component_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;

-- Declare IP libraries to ensure default binding in simulation. The IP library clause is ignored by synthesis.
LIBRARY ip_stratixiv_flash_lib;
LIBRARY ip_arria10_remote_update_altera_remote_update_141;

ENTITY tech_flash_remote_update IS
  GENERIC (
    g_technology : NATURAL := c_tech_select_default
  );
  PORT (
    clock       : IN STD_LOGIC;
    data_in     : IN STD_LOGIC_VECTOR (tech_flash_data_w(g_technology)-1 DOWNTO 0);
    param       : IN STD_LOGIC_VECTOR (2 DOWNTO 0);
    read_param  : IN STD_LOGIC;
    reconfig    : IN STD_LOGIC;
    reset       : IN STD_LOGIC;
    reset_timer : IN STD_LOGIC;
    write_param : IN STD_LOGIC;
    busy        : OUT STD_LOGIC;
    data_out    : OUT STD_LOGIC_VECTOR (tech_flash_data_w(g_technology)-1 DOWNTO 0)
  );
END tech_flash_remote_update;


ARCHITECTURE str OF tech_flash_remote_update IS

BEGIN

  gen_ip_stratixiv : IF g_technology=c_tech_stratixiv GENERATE
    u0 : ip_stratixiv_remote_update
    PORT MAP (clock, data_in, param, read_param, reconfig, reset, reset_timer, write_param, busy, data_out);
  END GENERATE;
  
  -- note 1: data_in and data_out must increase to 32 bits
  -- note 2: EPCQ-L1024 not yet supported in IP editor

  gen_ip_arria10 : IF g_technology=c_tech_arria10 GENERATE
    u0 : ip_arria10_remote_update
    PORT MAP (clock, data_in, param, read_param, reconfig, reset, reset_timer, write_param, busy, data_out);
  END GENERATE;
END ARCHITECTURE;
